/*

Let's toss in a few obstacles.

*/

area/Demo/Wall
	Enter(atom/movable/O)
		return FALSE

obj/Demo/Wanderer
	density				= TRUE

	New(Loc)
		..(Loc)
		walk_rand(src,20)

/*

You can define unique bounds for every atom/movable.

However, for this demo, we will go with a simple rectangle with the movable in the center.
Coordinates will be relative to the movable so x1 and y1 will be <= 0
while x2 and y2 are >= 0.

For example:
+-----------------------+
|				x2,y2	|
|		movable			|
|x1,y1					|
+-----------------------+

*/

atom/movable

	var
		x1	= 0
		y1	= 0
		x2	= 0
		y2	= 0

	//	We should know how to get our rectangle when standing on any turf.
	bb_PredictTurfs(turf/Loc,Dir)
		if ( !istype(Loc) )	return list()

		//	Restrict the turfs to within the bounds of the map.
		var
			X1	= max(1, min(Loc.x,Loc.x+src.x1))
			Y1	= max(1, min(Loc.y,Loc.y+src.y1))
			X2	= min(world.maxx, max(Loc.x,Loc.x+src.x2))
			Y2	= min(world.maxy, max(Loc.y,Loc.y+src.y2))

		//	BYOND containts a proc to get our rectangle of turfs.  Nifty, huh?
		return block( locate(X1,Y1,Loc.z), locate(X2,Y2,Loc.z) )


	//	Let's provide a way to update our bounds.
	//	I'll use this in some verbs behind the scenes, but you don't need to know that.
	proc/SetBounds(X1=0,Y1=0,X2=0,Y2=0)
		//	Check bounds.
		if ((X1 > 0) || (Y1 > 0))	return FALSE
		if ((X2 < 0) || (Y2 < 0))	return FALSE

		//	Find a turf so it can keep track of claims.
		var/turf/T	= locate(1,1,1)

		//	Remove old turfs.
		if ( istype(T) )	T.bb_AssignTurfs(src,null)

		//	Set new bounds.
		src.x1	= X1
		src.y1	= Y1
		src.x2	= X2
		src.y2	= Y2

		//	Claim new turfs.
		//	Note that I'm not bothering to check whether or not there is room in this simple demo.
		//	You'll probably want to make sure all of the predicted turfs can be entered.
		if ( istype(T) )	T.bb_AssignTurfs(src,src.bb_PredictTurfs(src.loc,src.dir))

		return TRUE

/*

I'm not going to do anything special here.
I just want to point out that there are multiple reasons for movement to fail.

*/

	//	This is the default value and our movables are single-tile objects by default anyway.
	//	I'm just showing it off so you know how the movables are first appearing on the map.
	bb_isSingleFromNull		= TRUE

	bb_BigBump(turf/NewLoc,list/Turfs)
		src						<< output("You can't enter [NewLoc] because:","bump")

		var
			turf/T
			atom/movable/AM
			list/AMs			= list()

		for(T in Turfs)

			//	Is the area the cause?
			//	turf.Enter() does not appear to call area.Enter() by default,
			//	but this proc will be sent turfs from restricted areas.
			if ( !T.loc.Enter(src) )
				src				<< output("\tYou bumped into a wall at [T].","bump")
				continue

			//	Look for dense movables.
			//	Note that it's possible to bump into the same movable in multiple places.
			for(AM in T.bb_GetWhatClaims())
				if (AM in AMs)	continue
				if (AM.density)
					src			<< output("\tYou bumped into [AM] at [T].","bump")
					AMs			+= AM
					break

/*

Something that is rather important is making sure to use this library's entered and exited procs.
You can ignore what I'm actually doing in these procs.
However, be sure to call these procs in your own project rather than turf.Entered() and turf.Exited().

*/

turf/Demo

	bb_EnteredBase(atom/movable/Obj,atom/OldLoc)
		if (!Obj.ignore_markExits)	flick("enter base",src)

	bb_EnteredPiece(atom/movable/Obj,atom/OldLoc)
		if (!Obj.ignore_markExits)	flick("enter piece",src)

	bb_ExitedBase(atom/movable/Obj)
		if (Obj.ignore_markExits)	flick("exit base",src)

	bb_ExitedPiece(atom/movable/Obj)
		if (Obj.ignore_markExits)	flick("exit piece",src)
