atom/movable

	//============================================================================================
	var
		bb_oldDir
		bb_isSingleFromNull		= TRUE
		bb_isSingleFromNonTurf	= TRUE

	//============================================================================================
	//--------------------------------------------------------------------------------------------
	Del()
		//	Release turfs.
		var/turf/T			= locate(1,1,1)
		if ( istype(T) )	T.bb_AssignTurfs(src,null)
		..()

	//--------------------------------------------------------------------------------------------
	Move(NewLoc,Dir)
		//	Keep track of last direction for predictions.
		src.bb_oldDir	= src.dir

		//	Bounding boxes only claim turfs.
		if ( !istype(NewLoc,/turf) )	return ..(NewLoc,Dir)

		//	Ignore blocked pieces when moving from null.
		if (!src.loc && src.bb_isSingleFromNull)	return ..(NewLoc,Dir)
		//	Ignore blocked pieces when moving from na non-turf.
		if (!istype(src.loc,/turf) && src.bb_isSingleFromNonTurf)	return ..(NewLoc,Dir)

		//	Make sure src can leave.
		var/atom/oldLoc						= src.loc
		if (oldLoc && !oldLoc.Exit(src))	return FALSE

		//	Find out what can not be entered.
		var/list/unenterables	= src.bb_GetBlockedTurfs(NewLoc,Dir)

		if (length(unenterables) <= 0)	return ..(NewLoc,Dir)

		//	BigBump!
		.	= FALSE
		src.bb_BigBump(NewLoc,unenterables)

	//============================================================================================
	proc
		//----------------------------------------------------------------------------------------
		//	This is the whole point of this library.
		//	What does it do by default?  Absolutely nothing!
		//	Turfs is every turf that could not be entered when the movable tried to move to NewLoc.
		//	There are multiple reasons for a movable not to be able to enter a turf.
		//	For instance, there could be density issues,
		//	or you could simply have disallowed such movement in your own code.
		//	Test for whatever the case is and react however you want.
		bb_BigBump(turf/NewLoc,list/Turfs)

		//----------------------------------------------------------------------------------------
		//	Returns all turfs assigned to src.
		bb_GetClaimedTurfs()
			var/turf/T		= locate(1,1,1)
			if (!istype(T))	return list()

			//	No claim?
			var/index		= T.bb_gTurfsPerMovable.Find(src)
			if (index <= 0)	return list()

			//	The stored list contains the movable followed by its turfs.
			//	Stop when a non-turf is reached.
			.				= list()
			index++
			while(index <= length(T.bb_gTurfsPerMovable))
				if ( !istype(T.bb_gTurfsPerMovable[index],/turf) )	break
				.	+= T.bb_gTurfsPerMovable[index]
				index++

		//----------------------------------------------------------------------------------------
		//	Returns all turfs that block src from entering Loc while facing Dir.
		//	By default, this returns any new turfs that fail their enter proc
		//	or are in areas that can't be entered.
		//	If this list has a positive length then Loc can not be entered
		//	and the list will be sent to bb_BigBump().
		bb_GetBlockedTurfs(turf/Loc,Dir)
			if ( !istype(Loc) )		return list()

			var
				list/unenterables	= src.bb_PredictTurfs(Loc,Dir) - src.bb_GetClaimedTurfs()
				turf/T
			for(T in unenterables)	if (T.Enter(src) && T.loc.Enter(src))	unenterables	-= T

			return unenterables

		//----------------------------------------------------------------------------------------
		//	Get all turfs assigned to a src as if it was standing at Loc.
		//	By default, if Loc was a turf then a list containing Loc is returned.
		//	If Loc was not a turf, then an empty list is returned.
		bb_PredictTurfs(turf/Loc,Dir)
			if ( !istype(Loc) )	return list()
			return list(Loc)
